/*****************************************************************************/
/*                             WiNRADiO G66WSB                               */
/*                                SDK DEMO                                   */
/*                                  FFT                                      */
/*                        (c) Copyright RADIXON 2025                         */
/*****************************************************************************/

#include "u_fft.h"
#include <math.h>
#include <string.h>

//The original of the following FFT can be found on the following link:
//http://www.drdobbs.com/cpp/a-simple-and-efficient-fft-implementatio/199500857?pgno=1
//The function does in-place FFT
//data - complex signal - data[0]=real0, data[1]=img0, data[2]=real1, data[3]=img1, ...
//nn - FFT size
void FFT(float* data, unsigned long nn)
{
    unsigned long n, mmax, m, j, istep, i;
    double wtemp, wr, wpr, wpi, wi, theta;
    double tempr, tempi;
    float t;

    // reverse-binary reindexing
    n = nn<<1;
    j=1;
    for (i=1; i<n; i+=2) {
        if (j>i) {
            t=data[j-1];
            data[j-1]=data[i-1];
            data[i-1]=t;

            t=data[j];
            data[j]=data[i];
            data[i]=t;
        }
        m = nn;
        while (m>=2 && j>m) {
            j -= m;
            m >>= 1;
        }
        j += m;
    };

    // here begins the Danielson-Lanczos section
    mmax=2;
    while (n>mmax) {
        istep = mmax<<1;
        theta = -(2*M_PI/mmax);
        wtemp = sin(0.5*theta);
        wpr = -2.0*wtemp*wtemp;
        wpi = sin(theta);
        wr = 1.0;
        wi = 0.0;
        for (m=1; m < mmax; m += 2) {
            for (i=m; i <= n; i += istep) {
                j=i+mmax;
                tempr = wr*data[j-1] - wi*data[j];
                tempi = wr * data[j] + wi*data[j-1];

                data[j-1] = (float)(data[i-1] - tempr);
                data[j] = (float)(data[i] - tempi);
                data[i-1]=(float)(data[i-1]+tempr);
                data[i]=(float)(data[i]+tempi);
            }
            wtemp=wr;
            wr += wr*wpr - wi*wpi;
            wi += wi*wpr + wtemp*wpi;
        }
        mmax=istep;
    }
}

//Computes Blackman window function coeffs and normalizes them for FFT
//http://en.wikipedia.org/wiki/Window_function
void GetNormalizedWindowCoeffs(float *Buffer,int Count,double FixedCoeff)
{
 double a0,a1,a2;
 int n;
 double Sum;

    a0=(1-0.16)/2;
    a1=0.5;
    a2=0.16/2;

    Sum=0;

    for(n=0;n<Count;n++)
    {
        Buffer[n]=(float)(a0-a1*cos((2*M_PI*n)/(Count-1))+a2*cos((4*M_PI*n)/(Count-1)));
        Sum+=Buffer[n];
    }

    Sum=FixedCoeff/Sum;

    for(n=0;n<Count;n++)
    {
        Buffer[n]=(float)(Buffer[n]*Sum);
    }
}
