/*****************************************************************************/
/*                             WiNRADiO G66WSB                               */
/*                                SDK DEMO                                   */
/*                             Main GUI window                               */
/*                        (c) Copyright RADIXON 2025                         */
/*****************************************************************************/

#include <wx/wupdlock.h>
#include "u_mainwindow.h"
#include "u_defs.h"
#include "u_fft.h"
#include "u_global.h"


enum
{
    ID_POWER=wxID_HIGHEST,
    ID_ATTENUATOR,
    ID_PREAMP,
    ID_CHANNEL_ACTIVITY,
    ID_DDC_SLEVEL_TIMER,
    ID_INIT_TIMER,
    ID_STATE_TIMER,
    ID_ACTIVE_DDC_ENABLED,
    ID_ACTIVE_DDC_CHANNEL,
    ID_ACTIVE_DDC_MUTE,
    ID_ACTIVE_DDC_VOLUME,
    ID_ACTIVE_DDC_AUDIO_GAIN,
    ID_AUDIO_SAMPLE_RATE,
    ID_DDC,
    ID_DDC_ENABLED=ID_DDC+32,
    ID_DDC_CHANNEL=ID_DDC_ENABLED+32,
};



BEGIN_EVENT_TABLE(TMainWindow, wxFrame)
    EVT_TOGGLEBUTTON(ID_POWER,TMainWindow::OnButtonClick)
    EVT_TOGGLEBUTTON(ID_ACTIVE_DDC_ENABLED,TMainWindow::OnButtonClick)
    EVT_TOGGLEBUTTON(ID_ACTIVE_DDC_MUTE,TMainWindow::OnButtonClick)
    EVT_TIMER(ID_DDC_SLEVEL_TIMER,TMainWindow::OnTimer)
    EVT_TIMER(ID_INIT_TIMER,TMainWindow::OnTimer)
    EVT_TIMER(ID_STATE_TIMER,TMainWindow::OnTimer)
    EVT_COMBOBOX(ID_ATTENUATOR,TMainWindow::OnComboBoxChanged)
    EVT_COMBOBOX(ID_ACTIVE_DDC_CHANNEL,TMainWindow::OnComboBoxChanged)
    EVT_COMBOBOX(ID_AUDIO_SAMPLE_RATE,TMainWindow::OnComboBoxChanged)
    EVT_TOGGLEBUTTON(ID_PREAMP,TMainWindow::OnButtonClick)
    EVT_CHANNEL_ACTIVITY_TUNE(ID_CHANNEL_ACTIVITY,TMainWindow::OnChannelActivityTune)
    EVT_NUMERICEDIT(ID_ACTIVE_DDC_VOLUME,TMainWindow::OnNumericEditChanged)
    EVT_NUMERICEDIT(ID_ACTIVE_DDC_AUDIO_GAIN,TMainWindow::OnNumericEditChanged)
END_EVENT_TABLE()

const wxString TMainWindow::m_AudioSampleRates[]=
{
    wxT("48 kHz"),  //G66WSB_AUDIO_SAMPLE_RATE_48000
    wxT("64 kHz"),  //G66WSB_AUDIO_SAMPLE_RATE_64000
    wxT("96 kHz"),  //G66WSB_AUDIO_SAMPLE_RATE_96000
    wxT("128 kHz"), //G66WSB_AUDIO_SAMPLE_RATE_128000
    wxT("192 kHz")  //G66WSB_AUDIO_SAMPLE_RATE_192000
};

TMainWindow::TMainWindow(IG66WSBDevice *Device):
    wxFrame(NULL,wxID_ANY,APP_TITLE,wxDefaultPosition,wxSize(900,800),wxCAPTION|wxCLOSE_BOX|wxSYSTEM_MENU|wxMINIMIZE_BOX),
    m_Device(Device),
    m_DDCSlevelTimer(this,ID_DDC_SLEVEL_TIMER),m_InitTimer(this,ID_INIT_TIMER),
    m_StateTimer(this,ID_STATE_TIMER)
{
 uint32_t i,j,k,Frequency;
 wxString Title,s;
 G66WSB_DEVICE_STATE DeviceState;
 wxSizer *MainSizer;
 wxSizer *BottomSizer,*TopSizer,*Sizer;
 wxGridSizer *GridSizer;
 uint32_t RowCount;
 float CompValue;
 uint32_t SonobuoyChannel;
 uint32_t AudioSampleRateIndex;
 wxString Channels[SONOBUOY_CHANNEL_COUNT];

    m_Device->GetDeviceInfo(&m_DeviceInfo);

    m_Device->GetAudioSampleRate(&AudioSampleRateIndex,&m_AudioSampleRate);

    if(m_DeviceInfo.MaxDDCCount>DDC_COUNT_LIMIT)
    {
        m_DDCCount=DDC_COUNT_LIMIT;
    }
    else
    {
        m_DDCCount=m_DeviceInfo.MaxDDCCount;
    }

    m_Connected=true;

    SetFont(*Global.m_Font);
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));

    Title.sprintf(APP_TITLE wxT(" - %s"),wxString(m_DeviceInfo.SerialNumber,wxConvLocal).c_str());

    switch(m_DeviceInfo.InterfaceType)
    {
        case G66WSB_INTERFACE_TYPE_USB2:
            Title+=wxT(" / USB2");
            break;

        case G66WSB_INTERFACE_TYPE_USB3:
            Title+=wxT(" / USB3");
            break;

        case G66WSB_INTERFACE_TYPE_PCIE:
            Title+=wxT(" / PCIe");
            break;

        default:
            break;
    }


    SetTitle(Title);
    SetDoubleBuffered(true);

    for(i=0;i<SONOBUOY_CHANNEL_COUNT;i++)
    {
        m_Device->GetChannelFrequency(i+1,&Frequency);
        Channels[i].sprintf(wxT("%02u - %.3f MHz"),SONOBUOY_CHANNEL_MIN+i,Frequency/1000000.0);
    }

    MainSizer=new wxBoxSizer(wxVERTICAL);

    RowCount=(m_DDCCount+3)/4;

    GridSizer=new wxGridSizer(RowCount,4,5,0);

    MainSizer->Add(GridSizer,0,wxEXPAND);

    m_DDCChannels=new struct TDDCChannel[m_DDCCount];
    m_DDCId2DDCIndex=new uint32_t[m_DDCCount];

    for(i=0;i<RowCount;i++)
    {
        for(j=0,k=i;j<4 && k<m_DDCCount;j++,k+=RowCount)
        {
            s.sprintf(wxT("DDC%u"),k);
            Sizer=new wxBoxSizer(wxHORIZONTAL);

            GridSizer->Add(Sizer);

            m_Device->CreateDDC(&m_DDCChannels[k].m_DDCId);

            m_DDCId2DDCIndex[m_DDCChannels[k].m_DDCId]=k;

            m_DDCChannels[k].m_BEnabled=new wxToggleButton(this,ID_DDC_ENABLED+k,s,wxDefaultPosition,wxSize(-1,CONTROL_HEIGHT));
            Sizer->Add(m_DDCChannels[k].m_BEnabled,0,wxALIGN_LEFT|wxALIGN_CENTRE_VERTICAL);

            m_DDCChannels[k].m_CBSonobuoyChannel=new wxComboBox(this,ID_DDC_CHANNEL+k,wxEmptyString,wxDefaultPosition,wxSize(-1,CONTROL_HEIGHT),SONOBUOY_CHANNEL_COUNT,Channels,wxCB_DROPDOWN|wxCB_READONLY|wxTE_CENTRE);
            Sizer->Add(m_DDCChannels[k].m_CBSonobuoyChannel,0,wxALIGN_LEFT|wxALIGN_CENTRE_VERTICAL);

            m_DDCChannels[k].m_CBSonobuoyChannel->SetSelection(k);

            SonobuoyChannel=k+SONOBUOY_CHANNEL_MIN;

            m_Device->SetChannel(m_DDCChannels[k].m_DDCId,SonobuoyChannel);
            m_Device->GetSpectrumCompensation(SonobuoyChannel,&CompValue);
            m_DDCChannels[k].m_SonobuoyChannel=SonobuoyChannel;
            m_DDCChannels[k].m_CompValue=CompValue;

            m_Device->SetVolume(m_DDCChannels[k].m_DDCId,20);

            m_DDCChannels[k].m_LSLevel=new wxStaticText(this,wxID_ANY,wxEmptyString,wxDefaultPosition,wxSize(60,-1),wxALIGN_RIGHT);
            Sizer->Add(m_DDCChannels[k].m_LSLevel,0,wxALIGN_CENTER_VERTICAL);
        }
    }

    Connect(ID_DDC_ENABLED,ID_DDC_ENABLED+m_DDCCount-1,wxEVT_TOGGLEBUTTON,wxCommandEventHandler(TMainWindow::OnDDCButtonClick));
    Connect(ID_DDC_CHANNEL,ID_DDC_CHANNEL+m_DDCCount-1,wxEVT_COMBOBOX,wxCommandEventHandler(TMainWindow::OnDDCComboBoxChanged));

    //DDC buttons
    m_ActiveDDCIndex=0;

    GridSizer=new wxGridSizer((m_DDCCount+15)/16,16,0,0);
    MainSizer->Add(GridSizer,0,wxEXPAND|wxTOP|wxBOTTOM,8);

    for(i=0;i<m_DDCCount;i++)
    {
        s.sprintf(wxT("DDC%u"),i);

        m_DDCChannels[i].m_BActive=new wxToggleButton(this,ID_DDC+i,s,wxDefaultPosition,wxSize(-1,CONTROL_HEIGHT));
        GridSizer->Add(m_DDCChannels[i].m_BActive,0,wxEXPAND);
    }

    m_DDCChannels[0].m_BActive->SetValue(true);

    Connect(ID_DDC,ID_DDC+m_DDCCount-1,wxEVT_TOGGLEBUTTON,wxCommandEventHandler(TMainWindow::OnActiveDDCButtonClick));

    //Spectrums
    GridSizer=new wxGridSizer(2,0,0);
    MainSizer->Add(GridSizer,1,wxEXPAND);

    //DDC spectrum
    Sizer=new wxBoxSizer(wxVERTICAL);
    GridSizer->Add(Sizer,1,wxEXPAND);

    TopSizer=new wxBoxSizer(wxHORIZONTAL);
    Sizer->Add(TopSizer,0,wxEXPAND|wxALIGN_TOP);

    m_BActiveDDC_Enabled=new wxToggleButton(this,ID_ACTIVE_DDC_ENABLED,wxT("Enabled"),wxDefaultPosition,wxSize(-1,CONTROL_HEIGHT));
    TopSizer->Add(m_BActiveDDC_Enabled);

    m_CBActiveDDC_SonobuoyChannel=new wxComboBox(this,ID_ACTIVE_DDC_CHANNEL,wxEmptyString,wxDefaultPosition,wxSize(-1,CONTROL_HEIGHT),SONOBUOY_CHANNEL_COUNT,Channels,wxCB_DROPDOWN|wxCB_READONLY|wxTE_CENTRE);
    TopSizer->Add(m_CBActiveDDC_SonobuoyChannel,0,wxALIGN_LEFT|wxALIGN_CENTRE_VERTICAL);
    m_CBActiveDDC_SonobuoyChannel->SetSelection(0);

    m_LActiveDDC_SLevel=new wxStaticText(this,wxID_ANY,wxEmptyString,wxDefaultPosition,wxSize(60,-1),wxALIGN_RIGHT);
    TopSizer->Add(m_LActiveDDC_SLevel,0,wxALIGN_CENTER_VERTICAL);

    m_DDCSpectrum=new TSpectrum(this,wxID_ANY);
    m_DDCSpectrum->SetRange(G66WSB_DDC_BANDWIDTH);
    m_DDCSpectrum->SetText(wxT("DDC spectrum"));
    Sizer->Add(m_DDCSpectrum,1,wxEXPAND);

    //Audio spectrum
    Sizer=new wxBoxSizer(wxVERTICAL);
    GridSizer->Add(Sizer,1,wxEXPAND);

    TopSizer=new wxBoxSizer(wxHORIZONTAL);
    Sizer->Add(TopSizer,0,wxEXPAND|wxALIGN_TOP);

    m_BActiveDDC_Mute=new wxToggleButton(this,ID_ACTIVE_DDC_MUTE,wxT("Mute"),wxDefaultPosition,wxSize(-1,CONTROL_HEIGHT));
    TopSizer->Add(m_BActiveDDC_Mute);

    m_LActiveDDC_Volume=new wxStaticText(this,wxID_ANY,wxT("Volume"));
    TopSizer->Add(m_LActiveDDC_Volume,0,wxALIGN_CENTER_VERTICAL|wxLEFT,4);

    m_EActiveDDC_Volume=new TIntEdit(this,ID_ACTIVE_DDC_VOLUME,20,0,31,wxDefaultPosition,wxSize(50,CONTROL_HEIGHT));
    TopSizer->Add(m_EActiveDDC_Volume,0,wxALIGN_CENTER_VERTICAL|wxLEFT,2);

    m_LActiveDDC_AudioGain=new wxStaticText(this,wxID_ANY,wxT("Gain"));
    TopSizer->Add(m_LActiveDDC_AudioGain,0,wxALIGN_CENTER_VERTICAL|wxLEFT,4);
    m_EActiveDDC_AudioGain=new TIntEdit(this,ID_ACTIVE_DDC_AUDIO_GAIN,0,-100,100,wxDefaultPosition,wxSize(50,CONTROL_HEIGHT));
    TopSizer->Add(m_EActiveDDC_AudioGain,0,wxALIGN_CENTER_VERTICAL|wxLEFT,2);
    m_LActiveDDC_AudioGain_dB=new wxStaticText(this,wxID_ANY,wxT("dB"));
    TopSizer->Add(m_LActiveDDC_AudioGain_dB,0,wxALIGN_CENTER_VERTICAL);

    m_LAudioSampleRate=new wxStaticText(this,wxID_ANY,wxT("Audio sample rate"));
    TopSizer->Add(m_LAudioSampleRate,0,wxALIGN_CENTER_VERTICAL|wxLEFT,16);
    m_CBAudioSampleRate=new wxComboBox(this,ID_AUDIO_SAMPLE_RATE,wxEmptyString,wxDefaultPosition,wxSize(-1,CONTROL_HEIGHT),sizeof(m_AudioSampleRates)/sizeof(m_AudioSampleRates[0]),m_AudioSampleRates,wxCB_DROPDOWN|wxCB_READONLY|wxTE_CENTRE);
    TopSizer->Add(m_CBAudioSampleRate,0,wxALIGN_CENTER_VERTICAL|wxLEFT,2);

    m_AudioSpecrum=new TSpectrum(this,wxID_ANY);
    m_AudioSpecrum->SetRange(m_AudioSampleRate/2);
    m_AudioSpecrum->SetCenterFrequency(m_AudioSampleRate/4);
    m_AudioSpecrum->SetText(wxT("Audio spectrum"));
    Sizer->Add(m_AudioSpecrum,1,wxEXPAND);

    //Channel activity
    m_ChannelActivity=new TChannelActivity(this,ID_CHANNEL_ACTIVITY);
    m_ChannelActivity->SetText(wxT("Channel activity"));
    MainSizer->Add(m_ChannelActivity,1,wxEXPAND);

    BottomSizer=new wxBoxSizer(wxHORIZONTAL);
    MainSizer->Add(BottomSizer,0,wxEXPAND|wxALIGN_BOTTOM);

    m_LAttenuator=new wxStaticText(this,wxID_ANY,wxT("Attenuator"));
    BottomSizer->Add(m_LAttenuator,0,wxALIGN_CENTER_VERTICAL);

    m_CBAttenuator=new wxComboBox(this,ID_ATTENUATOR,wxEmptyString,wxDefaultPosition,wxSize(-1,CONTROL_HEIGHT),0,0,wxCB_DROPDOWN|wxCB_READONLY|wxTE_CENTRE);
    BottomSizer->Add(m_CBAttenuator,0,wxALIGN_CENTER_VERTICAL|wxLEFT|wxRIGHT,4);

    m_CBAttenuator->Append(wxT("OFF"));

    for(i=1;i<=7;i++)
    {
        s.sprintf(wxT("%u dB"),i*3);
        m_CBAttenuator->Append(s);
    }

    m_CBAttenuator->SetSelection(0);

    m_BPreamp=new wxToggleButton(this,ID_PREAMP,wxT("Preamplifier"),wxDefaultPosition,wxSize(-1,CONTROL_HEIGHT));
    BottomSizer->Add(m_BPreamp,0,wxLEFT,4);

    BottomSizer->AddStretchSpacer();

    m_BPower=new wxToggleButton(this,ID_POWER,wxT("Power"),wxDefaultPosition,wxSize(-1,CONTROL_HEIGHT));
    BottomSizer->Add(m_BPower,0,wxALIGN_RIGHT);

    SetSizerAndFit(MainSizer);
    SetClientSize(-1,750);
    SetMinSize(wxSize(GetSize().GetWidth(),GetSize().GetHeight()));

    m_Device->SetCallback(this);

    Layout();
    Centre();

    m_Device->GetDeviceState(&DeviceState);

    UpdateState();

    SetActiveDDC(0);

    GetNormalizedWindowCoeffs(m_DDC_FFT_Coeffs,FFT_SIZE_DDC,1);
    GetNormalizedWindowCoeffs(m_Audio_FFT_Coeffs,FFT_SIZE_AUDIO,1);

    m_CBAudioSampleRate->Select(AudioSampleRateIndex);

    m_InitTimer.Start(100,true);
    m_StateTimer.Start(1000);
}

TMainWindow::~TMainWindow(void)
{
    m_Device->Close();
    m_Device->Free();

    delete[] m_DDCChannels;
    delete[] m_DDCId2DDCIndex;
}

void TMainWindow::G66WSB_DDCStreamCallback(IG66WSBDevice *Device,uint32_t DDCId,const float *Buffer,uint32_t Count,float Peak,float RMS)
{
 uint32_t DDCIndex,i;
 float *Coeffs;
 float *FFTBuffer;
 float m;
 float CompValue;

    DDCIndex=m_DDCId2DDCIndex[DDCId];

    m_DDCChannels[DDCIndex].m_SLevel=10.0f*log10(RMS*RMS*(1000.0f/50.0f)); //convert V to dBm

    if(m_ActiveDDCIndex==DDCIndex)
    {
        Coeffs=m_DDC_FFT_Coeffs;
        FFTBuffer=m_DDC_FFT_Buffer;

        for(i=0;i<FFT_SIZE_DDC;i++)
        {
            FFTBuffer[i*2]=Buffer[i*2]*Coeffs[i];       //I
            FFTBuffer[i*2+1]=Buffer[i*2+1]*Coeffs[i];   //Q
        }

        FFT(FFTBuffer,FFT_SIZE_DDC);

        //swap lower and upper half of the FFT result
        for(i=0;i<FFT_SIZE_DDC/2;i++)
        {
            std::swap(FFTBuffer[i*2],FFTBuffer[(i+FFT_SIZE_DDC/2)*2]);
            std::swap(FFTBuffer[i*2+1],FFTBuffer[(i+FFT_SIZE_DDC/2)*2+1]);
        }

        CompValue=m_DDCChannels[DDCIndex].m_CompValue;

        for(i=0;i<FFT_SIZE_DDC;i++)
        {
            m=FFTBuffer[i*2]*FFTBuffer[i*2]+FFTBuffer[i*2+1]*FFTBuffer[i*2+1];

            if(m>0)
            {
                FFTBuffer[i]=10*log10(m)+CompValue;
            }
            else
            {
                FFTBuffer[i]=-180;
            }
        }

        m_DDCSpectrum->SetData(FFTBuffer,FFT_SIZE_DDC,G66WSB_DDC_SAMPLE_RATE,(G66WSB_DDC_SAMPLE_RATE-G66WSB_DDC_BANDWIDTH)/2);
    }
}

void TMainWindow::G66WSB_AudioStreamCallback(IG66WSBDevice *Device,uint32_t DDCId,uint32_t Stage,const float *Buffer,uint32_t Count)
{
 uint32_t DDCIndex,i;
 float *Coeffs;
 float *FFTBuffer;
 float m;

    DDCIndex=m_DDCId2DDCIndex[DDCId];

    if(m_ActiveDDCIndex==DDCIndex)
    {
        switch(Stage)
        {
            case G66WSB_AUDIO_STREAM_CALLBACK_STAGE_1:
                Coeffs=m_Audio_FFT_Coeffs;
                FFTBuffer=m_Audio_FFT_Buffer;

                for(i=0;i<FFT_SIZE_AUDIO;i++)
                {
                    FFTBuffer[i*2]=Buffer[i]*Coeffs[i];
                    FFTBuffer[i*2+1]=0;
                }

                FFT(FFTBuffer,FFT_SIZE_AUDIO);

                for(i=0;i<FFT_SIZE_AUDIO/2;i++)
                {
                    m=FFTBuffer[i*2]*FFTBuffer[i*2]+FFTBuffer[i*2+1]*FFTBuffer[i*2+1];

                    if(m>0)
                    {
                        FFTBuffer[i]=10*log10(m);
                    }
                    else
                    {
                        FFTBuffer[i]=-180;
                    }
                }

                m_AudioSpecrum->SetData(FFTBuffer,FFT_SIZE_AUDIO/2,m_AudioSampleRate/2,0);
                break;

            case G66WSB_AUDIO_STREAM_CALLBACK_STAGE_2:
                m_WaveOut.Write(Buffer,Count);
                break;

            default:
                break;
        }
    }
}

void TMainWindow::G66WSB_ChannelActivityCallback(IG66WSBDevice *Device,const float *RMS,const float *dBm,uint16_t ADCLevel)
{
    m_ChannelActivity->SetChannels(dBm);
}

void TMainWindow::OnButtonClick(wxCommandEvent &Event)
{
 wxBusyCursor *BusyCursor;

    switch(Event.GetId())
    {
        case ID_POWER:
            BusyCursor=new wxBusyCursor;
            m_BPower->Update();
            SetPower(m_BPower->GetValue());
            delete BusyCursor;
            break;

        case ID_PREAMP:
            m_Device->SetPreamplifier(m_BPreamp->GetValue());
            UpdateCompValues();
            break;

        case ID_ACTIVE_DDC_ENABLED:
            if(m_BActiveDDC_Enabled->GetValue())
            {
                StartDDC(m_ActiveDDCIndex);
            }
            else
            {
                StopDDC(m_ActiveDDCIndex);
            }
            UpdateActiveDDCState();
            break;

        case ID_ACTIVE_DDC_MUTE:
            m_Device->SetMute(m_DDCChannels[m_ActiveDDCIndex].m_DDCId,m_BActiveDDC_Mute->GetValue());
            break;
    }
}

void TMainWindow::SetPower(BOOL Power)
{
 uint32_t i;
 wxWindowUpdateLocker Locker(this);

    m_Device->SetPower(Power);
    m_Device->GetPower(&Power);

    if(Power)
    {
        m_Device->StartChannelActivity(UPDATE_INTERVAL);
        m_DDCSlevelTimer.Start(100);
        m_WaveOut.Open(m_AudioSampleRate);
    }
    else
    {
        m_ChannelActivity->Clear();
        m_DDCSlevelTimer.Stop();
        m_WaveOut.Close();

        for(i=0;i<m_DDCCount;i++)
        {
            m_DDCChannels[i].m_BEnabled->SetValue(false);
            m_DDCChannels[i].m_LSLevel->SetLabel(wxEmptyString);
        }

        m_BActiveDDC_Enabled->SetValue(false);
    }

    m_BPower->SetValue(Power!=FALSE);

    UpdateState();
}

void TMainWindow::UpdateState(void)
{
 bool Enable;
 uint32_t i;

    if(m_Connected)
    {
        Enable=m_BPower->GetValue();
    }
    else
    {
        Enable=false;
        m_BPower->SetValue(false);
        m_BPower->Enable(false);
    }

    m_LAttenuator->Enable(Enable);
    m_CBAttenuator->Enable(Enable);

    m_BPreamp->Enable(Enable);

    for(i=0;i<m_DDCCount;i++)
    {
        m_DDCChannels[i].SetEnabled(Enable);
    }

    m_ChannelActivity->Enable(Enable);

    m_BActiveDDC_Enabled->Enable(Enable);
    m_CBActiveDDC_SonobuoyChannel->Enable(Enable);
    m_BActiveDDC_Mute->Enable(Enable);
    m_LActiveDDC_Volume->Enable(Enable);
    m_EActiveDDC_Volume->Enable(Enable);
    m_LActiveDDC_AudioGain->Enable(Enable);
    m_LActiveDDC_AudioGain_dB->Enable(Enable);
    m_EActiveDDC_AudioGain->Enable(Enable);

    UpdateActiveDDCState();
}

void TMainWindow::UpdateActiveDDCState(void)
{
 bool Enable;

    Enable=m_BPower->GetValue() && m_DDCChannels[m_ActiveDDCIndex].m_BEnabled->GetValue() && m_Connected;

    m_DDCSpectrum->Enable(Enable);
    m_AudioSpecrum->Enable(Enable);

    m_LActiveDDC_SLevel->SetLabel(wxEmptyString);
}

void TMainWindow::StartDDC(uint32_t DDCIndex)
{
 uint32_t DDCSampleSetsPerBuffer;
 uint32_t AudioSamplesPerBuffer;

    m_DDCChannels[DDCIndex].m_BEnabled->SetValue(true);

    if(DDCIndex==m_ActiveDDCIndex)
    {
        m_BActiveDDC_Enabled->SetValue(true);
    }

    DDCSampleSetsPerBuffer=ALING_SAMPLE_COUNT(G66WSB_DDC_SAMPLE_RATE*UPDATE_INTERVAL/1000);
    AudioSamplesPerBuffer=ALING_SAMPLE_COUNT(m_AudioSampleRate*UPDATE_INTERVAL/1000);

    m_Device->StartDDC(m_DDCChannels[DDCIndex].m_DDCId,DDCSampleSetsPerBuffer);
    m_Device->StartAudio(m_DDCChannels[DDCIndex].m_DDCId,AudioSamplesPerBuffer);
}

void TMainWindow::StopDDC(uint32_t DDCIndex)
{
    //m_Device->StopAudio(m_DDCChannels[DDCIndex].m_DDCId); this is not required, the StopDDC stops audio, as well
    m_Device->StopDDC(m_DDCChannels[DDCIndex].m_DDCId);

    m_DDCChannels[DDCIndex].m_LSLevel->SetLabel(wxEmptyString);

    m_DDCChannels[DDCIndex].m_BEnabled->SetValue(false);

    if(m_ActiveDDCIndex==DDCIndex)
    {
        m_DDCSpectrum->Clear();
        m_AudioSpecrum->Clear();
        m_BActiveDDC_Enabled->SetValue(false);
    }
}

void TMainWindow::UpdateCompValues(void)
{
 uint32_t i;
 float CompValue;

    for(i=0;i<m_DDCCount;i++)
    {
        m_Device->GetSpectrumCompensation(m_DDCChannels[i].m_SonobuoyChannel,&CompValue);
        m_DDCChannels[i].m_CompValue=CompValue;
    }
}

void TMainWindow::OnComboBoxChanged(wxCommandEvent &Event)
{
    switch(Event.GetId())
    {
        case ID_ATTENUATOR:
            m_Device->SetAttenuator(m_CBAttenuator->GetSelection()*3);
            UpdateCompValues();
            break;

        case ID_ACTIVE_DDC_CHANNEL:
            SetChannel(m_ActiveDDCIndex,m_CBActiveDDC_SonobuoyChannel->GetSelection()+SONOBUOY_CHANNEL_MIN);
            break;

        case ID_AUDIO_SAMPLE_RATE:
            SetAudioSampleRate(m_CBAudioSampleRate->GetSelection());
            break;
    }
}

void TMainWindow::OnDDCButtonClick(wxCommandEvent &Event)
{
 uint32_t DDCIndex;

    DDCIndex=Event.GetId()-ID_DDC_ENABLED;

    if(m_DDCChannels[DDCIndex].m_BEnabled->GetValue())
    {
        StartDDC(DDCIndex);
    }
    else
    {
        StopDDC(DDCIndex);
    }

    UpdateActiveDDCState();
}

void TMainWindow::SetActiveDDC(uint32_t DDCIndex)
{
 uint32_t Frequency;
 uint32_t SonobuoyChannel;
 uint8_t Volume;
 BOOL Mute;
 double Gain;

    m_ActiveDDCIndex=DDCIndex;
    m_Device->GetChannel(m_DDCChannels[DDCIndex].m_DDCId,&SonobuoyChannel,&Frequency);
    m_Device->GetVolume(m_DDCChannels[DDCIndex].m_DDCId,&Volume);
    m_Device->GetMute(m_DDCChannels[DDCIndex].m_DDCId,&Mute);
    m_Device->GetAudioGain(m_DDCChannels[DDCIndex].m_DDCId,&Gain);

    m_DDCSpectrum->SetCenterFrequency(Frequency);

    m_CBActiveDDC_SonobuoyChannel->SetSelection(SonobuoyChannel-SONOBUOY_CHANNEL_MIN);
    m_BActiveDDC_Enabled->SetValue(m_DDCChannels[DDCIndex].m_BEnabled->GetValue());

    m_BActiveDDC_Mute->SetValue(Mute!=FALSE);
    m_EActiveDDC_Volume->Set(Volume);
    m_EActiveDDC_AudioGain->Set(Gain);

    UpdateActiveDDCState();
}

void TMainWindow::OnActiveDDCButtonClick(wxCommandEvent &Event)
{
 uint32_t DDCIndex,i;

    DDCIndex=Event.GetId()-ID_DDC;

    if(m_DDCChannels[DDCIndex].m_BActive->GetValue())
    {
        SetActiveDDC(DDCIndex);

        for(i=0;i<m_DDCCount;i++)
        {
            if(i!=DDCIndex)
            {
                m_DDCChannels[i].m_BActive->SetValue(false);
            }
        }
    }
    else
    {
        m_DDCChannels[DDCIndex].m_BActive->SetValue(true);
    }
}

void TMainWindow::OnDDCComboBoxChanged(wxCommandEvent &Event)
{
 uint32_t DDCIndex;
 uint32_t SonobuoyChannel;


    DDCIndex=Event.GetId()-ID_DDC_CHANNEL;

    SonobuoyChannel=m_DDCChannels[DDCIndex].m_CBSonobuoyChannel->GetSelection()+SONOBUOY_CHANNEL_MIN;

    SetChannel(DDCIndex,SonobuoyChannel);
}

void TMainWindow::SetChannel(uint32_t DDCIndex,uint32_t SonobuoyChannel)
{
 float CompValue;
 uint32_t Frequency;

    m_DDCChannels[DDCIndex].m_CBSonobuoyChannel->SetSelection(SonobuoyChannel-1);

    m_Device->SetChannel(m_DDCChannels[DDCIndex].m_DDCId,SonobuoyChannel);

    m_Device->GetSpectrumCompensation(SonobuoyChannel,&CompValue);

    m_DDCChannels[DDCIndex].m_CompValue=CompValue;
    m_DDCChannels[DDCIndex].m_SonobuoyChannel=SonobuoyChannel;

    if(m_ActiveDDCIndex==DDCIndex)
    {
        m_Device->GetChannel(m_DDCChannels[DDCIndex].m_DDCId,NULL,&Frequency);

        m_DDCSpectrum->SetCenterFrequency(Frequency);

        m_CBActiveDDC_SonobuoyChannel->SetSelection(SonobuoyChannel-1);
    }
}

void TMainWindow::OnTimer(wxTimerEvent& Event)
{
 uint32_t i;
 wxString s;
 BOOL Connected;
 G66WSB_DEVICE_STATE DeviceState;

    switch(Event.GetId())
    {
        case ID_DDC_SLEVEL_TIMER:
            for(i=0;i<m_DDCCount;i++)
            {
                if(m_DDCChannels[i].m_BEnabled->GetValue())
                {
                    s.sprintf(wxT("%.1f dBm"),m_DDCChannels[i].m_SLevel.load());
                    m_DDCChannels[i].m_LSLevel->SetLabel(s);

                    if(i==m_ActiveDDCIndex)
                    {
                        m_LActiveDDC_SLevel->SetLabel(s);
                    }
                }
            }
            break;

        case ID_STATE_TIMER:
            m_Device->IsConnected(&Connected);

            if(!Connected)
            {
                m_Connected=false;

                m_StateTimer.Stop();
                m_DDCSlevelTimer.Stop();

                UpdateState();

                wxMessageBox(APP_TITLE wxT(" will be closed.\nReceiver is no longer connected."),wxT("Warning"),wxICON_WARNING|wxOK);
                Close();
                break;
            }

            if(m_Device->GetDeviceState(&DeviceState))
            {
                if(m_BPower->GetValue())
                {
                    if(DeviceState.Flags & G66WSB_DEVICE_STATE_HIGH_TEMPERATURE)
                    {
                        SetPower(FALSE);

                        wxMessageBox(wxT("WARNING: The receiver's temperature is too high.\n")
                                 wxT("The receiver has been turned off to prevent damage.\n\n")
                                 wxT("Please wait a moment and then turn the receiver on again.\n")
                                 wxT("If the problem persists, please ensure that the receiver's fan\n")
                                 wxT("and free air flow around the receiver are not obstructed."),wxT("Warning"),wxICON_WARNING|wxOK);
                    }
                }
            }
            break;

        case ID_INIT_TIMER:
            SetPower(TRUE);
            StartDDC(0);
            UpdateActiveDDCState();
            break;
    }
}

void TMainWindow::OnChannelActivityTune(TChannelActivityTuneEvent &Event)
{
 uint32_t SonobuoyChannel;

    SonobuoyChannel=Event.GetChannel();

    SetChannel(m_ActiveDDCIndex,SonobuoyChannel);
}

void TMainWindow::OnNumericEditChanged(wxNotifyEvent &Event)
{
    switch(Event.GetId())
    {
        case ID_ACTIVE_DDC_VOLUME:
            m_Device->SetVolume(m_DDCChannels[m_ActiveDDCIndex].m_DDCId,(uint8_t)m_EActiveDDC_Volume->Get());
            break;

        case ID_ACTIVE_DDC_AUDIO_GAIN:
            m_Device->SetAudioGain(m_DDCChannels[m_ActiveDDCIndex].m_DDCId,(int)m_EActiveDDC_AudioGain->Get());
            break;

        default:
            break;
    }

}


void TMainWindow::SetAudioSampleRate(uint32_t Index)
{
 uint32_t i;
 uint32_t AudioSampleRateIndex;
 int result;


    if(m_BPower->GetValue())
    {
        for(i=0;i<m_DDCCount;i++)
        {
            m_DDCChannels[i].m_Restart=m_DDCChannels[i].m_BEnabled->GetValue();
            StopDDC(i);
        }

        m_WaveOut.Close();
    }

    result=m_Device->SetAudioSampleRate(Index);

    m_Device->GetAudioSampleRate(&AudioSampleRateIndex,&m_AudioSampleRate);

    if(!result)
    {
        m_CBAudioSampleRate->SetSelection(AudioSampleRateIndex);
    }

    m_AudioSpecrum->Clear();
    m_AudioSpecrum->SetRange(m_AudioSampleRate/2);
    m_AudioSpecrum->SetCenterFrequency(m_AudioSampleRate/4);

    if(m_BPower->GetValue())
    {
        for(i=0;i<m_DDCCount;i++)
        {
            if(m_DDCChannels[i].m_Restart)
            {
                StartDDC(i);
            }
        }

        m_WaveOut.Open(m_AudioSampleRate);
    }
}

