#include "g66wsbapi.h"
#include <stdio.h>
#include <math.h>
#include <dlfcn.h>
#include <inttypes.h>
#include <pthread.h>
#include <pulse/simple.h>

#define AUDIO_SAMPLE_RATE 64000

G66WSB_OPEN_DEVICE OpenDevice;
G66WSB_CLOSE_DEVICE CloseDevice;
G66WSB_SET_POWER SetPower;
G66WSB_CREATE_DDC CreateDDC;
G66WSB_SET_CALLBACKS SetCallbacks;
G66WSB_SET_CHANNEL SetChannel;
G66WSB_GET_CHANNEL GetChannel;
G66WSB_START_DDC StartDDC;
G66WSB_STOP_DDC StopDDC;
G66WSB_START_AUDIO StartAudio;
G66WSB_STOP_AUDIO StopAudio;
G66WSB_SET_AUDIO_SAMPLE_RATE SetAudioSampleRate;

int32_t hDevice;
void *API=NULL;
pa_simple *PaSimple=NULL;

int Initialize(void)
{
 pa_sample_spec SampleFormat;
 int Error;

    API=dlopen("libg66wsbapi.so",RTLD_LAZY);

    if(API==NULL)
    {
        printf("Failed to load libg66wsbapi.so. %s\n",dlerror());
        return 0;
    }

    //retrieve addresses of used API functions
    OpenDevice=(G66WSB_OPEN_DEVICE)dlsym(API,"OpenDevice");
    CloseDevice=(G66WSB_CLOSE_DEVICE)dlsym(API,"CloseDevice");
    SetPower=(G66WSB_SET_POWER)dlsym(API,"SetPower");
    CreateDDC=(G66WSB_CREATE_DDC)dlsym(API,"CreateDDC");
    SetCallbacks=(G66WSB_SET_CALLBACKS)dlsym(API,"SetCallbacks");
    SetChannel=(G66WSB_SET_CHANNEL)dlsym(API,"SetChannel");
    GetChannel=(G66WSB_GET_CHANNEL)dlsym(API,"GetChannel");
    StartDDC=(G66WSB_START_DDC)dlsym(API,"StartDDC");
    StopDDC=(G66WSB_STOP_DDC)dlsym(API,"StopDDC");
    StartAudio=(G66WSB_START_AUDIO)dlsym(API,"StartAudio");
    StopAudio=(G66WSB_STOP_AUDIO)dlsym(API,"StopAudio");
    SetAudioSampleRate=(G66WSB_SET_AUDIO_SAMPLE_RATE)dlsym(API,"SetAudioSampleRate");

    //open the first available G66WSB device
    hDevice=OpenDevice(G66WSB_OPEN_FIRST);

    if(hDevice<0)
    {
        printf("No available G66WSB device found.\n");
        return 0;
    }

    //initialize pulse audio
    SampleFormat.channels=1;
    SampleFormat.rate=AUDIO_SAMPLE_RATE;
    SampleFormat.format=PA_SAMPLE_FLOAT32;

    PaSimple=pa_simple_new(NULL,"G66WSB SDK EXAMPLE",PA_STREAM_PLAYBACK,NULL,"Audio0",&SampleFormat,NULL,NULL,&Error);

    if(PaSimple==NULL)
    {
        printf("Failed to initialize pulse audio.\n");
    }

    return 1;
}

void MyDDCStreamCallback(uint32_t DDCId,const float *Buffer,uint32_t Count,float SlevelPeak,float SlevelRMS,uintptr_t UserData)
{
 double Slevel_dBm;

    //convert slevel RMS in Volts to dBm
    Slevel_dBm=10.0*log10(SlevelRMS*SlevelRMS*(1000.0/50.0));

    printf("\rSlevel: RMS[V]=%9.7f V, RMS[dBm]=%6.1f dBm, Peak[V]=%9.7f V",SlevelRMS,Slevel_dBm,SlevelPeak);
}

void MyAudioStreamCallback(uint32_t DDCId,uint32_t Stage,const float *Buffer,uint32_t Count,uintptr_t UserData)
{
 int Error;

    if(Stage==2 && PaSimple!=NULL)
    {
        //we want to play only audio buffers of type 2 which means final audio samples (audio gain, audio filter and volume are applied)
        pa_simple_write(PaSimple,Buffer,Count*sizeof(float),&Error);
    }
}

int main(void)
{
 G66WSB_CALLBACKS Callbacks;
 uint32_t DDCId;
 uint32_t SonobuoyChannel;
 uint32_t Frequency;

    if(!Initialize())
    {
        return -1;
    }

    //create DDC
    CreateDDC(hDevice,&DDCId);

    //set power
    SetPower(hDevice,1);

    //set audio sample rate to 64 kHz
    SetAudioSampleRate(hDevice,G66WSB_AUDIO_SAMPLE_RATE_64000);

    //ChannelActivityCallback is not required in this example
    Callbacks.ChannelActivityCallback=NULL;

    //DDCStreamCallback is used to display signal level in this example
    Callbacks.DDCStreamCallback=MyDDCStreamCallback;

    //AudioStreamCallback is used to play audio on wave out device
    Callbacks.AudioStreamCallback=MyAudioStreamCallback;

    //register callback functiobs
    SetCallbacks(hDevice,&Callbacks,0);

    //tune to Sonobuoy channel 13
    SetChannel(hDevice,DDCId,13);

    GetChannel(hDevice,DDCId,&SonobuoyChannel,&Frequency);

    printf("Listening to sonobuoy channel %u (%u Hz).\n",SonobuoyChannel,Frequency);

    //start DDC streaming
    StartDDC(hDevice,DDCId,8192);

    //start audio streaming, 2048 samples per buffer
    StartAudio(hDevice,DDCId,2048);

    printf("\nPress ENTER to exit.\n\n");
    getchar();

    //stop audio streaming
    StopAudio(hDevice,DDCId);

    //stop DDC streaming
    StopDDC(hDevice,DDCId);

    //unregister all callback functions
    SetCallbacks(hDevice,NULL,0);

    SetPower(hDevice,0);

    CloseDevice(hDevice);

    if(PaSimple)
    {
        pa_simple_free(PaSimple);
    }

    dlclose(API);

    printf("\n");

    return 0;
}
