#ifndef __G66WSBAPI_H__
#define __G66WSBAPI_H__

#include <stdint.h>

#pragma pack(push,1)

#define G66WSB_CLASS_ID_DEVICE_ENUMERATOR	            0x0066F901
#define G66WSB_CLASS_ID_DEVICE		                    0x0066F902

typedef int (*G66WSB_CREATE_INSTANCE)(uint32_t ClassId,void **Interface);

#define G66WSB_OPEN_DEMO                                ((char*)0)
#define G66WSB_OPEN_FIRST                               ((char*)-1)

#define G66WSB_INTERFACE_TYPE_PCIE                      0x00
#define G66WSB_INTERFACE_TYPE_USB2                      0x01
#define G66WSB_INTERFACE_TYPE_USB3                      0x02
#define G66WSB_INTERFACE_TYPE_DEMO                      0xFF

#define G66WSB_FRONT_PANEL_LED_MODE_OFF                 0
#define G66WSB_FRONT_PANEL_LED_MODE_ON                  1
#define G66WSB_FRONT_PANEL_LED_MODE_DIAG                2

#define G66WSB_RANGE_1                                  0
#define G66WSB_RANGE_2                                  1
#define G66WSB_RANGE_COUNT                              2

#define G66WSB_DEVICE_STATE_HIGH_TEMPERATURE            0x00000001

#define G66WSB_TEMPERATURE_UNKNOWN                      -16384

#define G66WSB_DDC_SAMPLE_RATE                          312500
#define G66WSB_DDC_BANDWIDTH                            250000
#define G66WSB_AUDIO_SAMPLE_RATE_48000                  0
#define G66WSB_AUDIO_SAMPLE_RATE_64000                  1
#define G66WSB_AUDIO_SAMPLE_RATE_96000                  2
#define G66WSB_AUDIO_SAMPLE_RATE_128000                 3
#define G66WSB_AUDIO_SAMPLE_RATE_192000                 4

#define G66WSB_AUDIO_STREAM_CALLBACK_STAGE_0            0
#define G66WSB_AUDIO_STREAM_CALLBACK_STAGE_1            1
#define G66WSB_AUDIO_STREAM_CALLBACK_STAGE_2            2

typedef struct
{
    char        DevicePath[256];
    uint8_t     InterfaceType;
    char        SerialNumber[9];
    uint16_t    HWVersion;
    uint16_t    FWVersion[3];
    uint8_t     EEPROMVersion;
    uint32_t    MaxDDCCount;
    uint32_t    Flags;
} G66WSB_DEVICE_INFO;

typedef struct
{
    uint32_t    Flags;
    int32_t     Temperatures[3];
    uint32_t    FanRPM;
    uint64_t    DataTransferred;
    uint64_t    DataLost;
} G66WSB_DEVICE_STATE;


#ifdef __cplusplus

#define G66WSB_DECLARE_INTERFACE(__name__) struct __name__
#define G66WSB_DECLARE_INTERFACE_2(__name__,__base__) struct __name__ : public __base__
#define G66WSB_THIS
#define G66WSB_METHOD(__type__,__method__,__params__) virtual __type__ __method__ __params__=0
#define G66WSB_METHOD_NO_PARAMS(__type__,__method__) virtual __type__ __method__(void)=0
#else //__cplusplus
#define G66WSB_THIS struct G66WSB_INTERFACE *This,
#define G66WSB_DECLARE_INTERFACE(__name__) typedef struct __name__ { const struct __name__##Vtbl *lpVtbl;} __name__;\
    typedef const struct __name__##Vtbl __name__##Vtbl;struct __name__##Vtbl
#define G66WSB_DECLARE_INTERFACE_2(__name__,__base__) G66WSB_DECLARE_INTERFACE(__name__)
#define G66WSB_METHOD(__type__,__method__,__params__) __type__ (*__method__)__params__
#define G66WSB_METHOD_NO_PARAMS(__type__,__method__) __type__ (*__method__)(struct G66WSB_INTERFACE *This)
#endif //__cplusplus

#undef G66WSB_INTERFACE
#define G66WSB_INTERFACE IG66WSBDeviceEnumerator

G66WSB_DECLARE_INTERFACE(IG66WSBDeviceEnumerator)
{
    G66WSB_METHOD_NO_PARAMS(void,Free);
    G66WSB_METHOD_NO_PARAMS(int,Enumerate);
    G66WSB_METHOD_NO_PARAMS(uint32_t,GetCount);
    G66WSB_METHOD(int,GetDeviceInfo,(G66WSB_THIS uint32_t DeviceIndex,G66WSB_DEVICE_INFO *DeviceInfo));
};

struct IG66WSBDevice;

#undef G66WSB_INTERFACE
#define G66WSB_INTERFACE IG66WSBDeviceCallback

G66WSB_DECLARE_INTERFACE(IG66WSBDeviceCallback)
{
    G66WSB_METHOD(void,G66WSB_DDCStreamCallback,(G66WSB_THIS struct IG66WSBDevice *Device,uint32_t DDCId,const float *Buffer,uint32_t Count,float Peak,float RMS));
    G66WSB_METHOD(void,G66WSB_AudioStreamCallback,(G66WSB_THIS struct IG66WSBDevice *Device,uint32_t DDCId,uint32_t Stage,const float *Buffer,uint32_t Count));
    G66WSB_METHOD(void,G66WSB_ChannelActivityCallback,(G66WSB_THIS struct IG66WSBDevice *Device,const float *RMS,const float *dBm,uint16_t ADCLevel));
};

#define G66WSB_DEVICE_INTERFACE_VERSION         0x00000002

#undef G66WSB_INTERFACE
#define G66WSB_INTERFACE IG66WSBDevice

G66WSB_DECLARE_INTERFACE(IG66WSBDevice)
{
    G66WSB_METHOD_NO_PARAMS(int,Free);
    G66WSB_METHOD(int,GetInterfaceVersion,(G66WSB_THIS uint32_t *Version));

    G66WSB_METHOD(void,SetCallback,(G66WSB_THIS IG66WSBDeviceCallback *Callback));
    G66WSB_METHOD_NO_PARAMS(IG66WSBDeviceCallback*,GetCallback);

    G66WSB_METHOD(int,Open,(G66WSB_THIS const char *Path));
    G66WSB_METHOD_NO_PARAMS(int,Close);

    G66WSB_METHOD_NO_PARAMS(int,IsOpen);

    G66WSB_METHOD(int,GetDeviceInfo,(G66WSB_THIS G66WSB_DEVICE_INFO *DeviceInfo));

    G66WSB_METHOD(int,IsConnected,(G66WSB_THIS int *Connected));

    G66WSB_METHOD(int,SetLED,(G66WSB_THIS uint32_t LEDMode));
    G66WSB_METHOD(int,GetLED,(G66WSB_THIS uint32_t *LEDMode));

    G66WSB_METHOD(int,SetPower,(G66WSB_THIS int Power));
    G66WSB_METHOD(int,GetPower,(G66WSB_THIS int *Power));

    G66WSB_METHOD(int,GetDeviceState,(G66WSB_THIS G66WSB_DEVICE_STATE *State));

    G66WSB_METHOD(int,SetBuiltInTest,(G66WSB_THIS int Enabled));
    G66WSB_METHOD(int,GetBuiltInTest,(G66WSB_THIS int *Enabled));

    G66WSB_METHOD(int,SetAttenuator,(G66WSB_THIS uint32_t Attenuator));
    G66WSB_METHOD(int,GetAttenuator,(G66WSB_THIS uint32_t *Attenuator));

    G66WSB_METHOD(int,SetPreamplifier,(G66WSB_THIS int Preamp));
    G66WSB_METHOD(int,GetPreamplifier,(G66WSB_THIS int *Preamp));

    G66WSB_METHOD(int,SetADCNoiseBlanker,(G66WSB_THIS int Enabled));
    G66WSB_METHOD(int,GetADCNoiseBlanker,(G66WSB_THIS int *Enabled));
    G66WSB_METHOD(int,SetADCNoiseBlankerThreshold,(G66WSB_THIS uint16_t Threshold));
    G66WSB_METHOD(int,GetADCNoiseBlankerThreshold,(G66WSB_THIS uint16_t *Threshold) );

    G66WSB_METHOD(int,CreateDDC,(G66WSB_THIS uint32_t *DDCId));
    G66WSB_METHOD(int,DeleteDDC,(G66WSB_THIS uint32_t DDCId));

    G66WSB_METHOD(int,StartDDC,(G66WSB_THIS uint32_t DDCId,uint32_t SampleSetsPerBuffer));
    G66WSB_METHOD(int,StopDDC,(G66WSB_THIS uint32_t DDClId));

    G66WSB_METHOD(int,SetChannel,(G66WSB_THIS uint32_t DDCId,uint32_t Channel));
    G66WSB_METHOD(int,GetChannel,(G66WSB_THIS uint32_t DDCId,uint32_t *Channel,uint32_t *Frequency));

    G66WSB_METHOD(int,GetChannelFrequency,(G66WSB_THIS uint32_t Channel,uint32_t *Frequency));

    G66WSB_METHOD(int,GetSignalLevel,(G66WSB_THIS uint32_t DDCId,float *Peak,float *RMS));

    G66WSB_METHOD(int,StartAudio,(G66WSB_THIS uint32_t DDCId,uint32_t SampleSetsPerBuffer));
    G66WSB_METHOD(int,StopAudio,(G66WSB_THIS uint32_t DDCId));

    G66WSB_METHOD(int,SetAudioFilter,(G66WSB_THIS uint32_t DDCId,int Enabled));
    G66WSB_METHOD(int,GetAudioFilter,(G66WSB_THIS uint32_t DDCId,int *Enabled));
    G66WSB_METHOD(int,SetAudioFilterParams,(G66WSB_THIS uint32_t DDCId,uint32_t CutOffLow,uint32_t CutOffHigh,double Deemphasis));
    G66WSB_METHOD(int,GetAudioFilterParams,(G66WSB_THIS uint32_t DDCId,uint32_t *CutOffLow,uint32_t *CutOffHigh,double *Deemphasis));
    G66WSB_METHOD(int,SetAudioFilterLength,(G66WSB_THIS uint32_t DDCId,uint32_t Length));
    G66WSB_METHOD(int,GetAudioFilterLength,(G66WSB_THIS uint32_t DDCId,uint32_t *Length));
    G66WSB_METHOD(int,SetAudioGain,(G66WSB_THIS uint32_t DDCId,double Gain));
    G66WSB_METHOD(int,GetAudioGain,(G66WSB_THIS uint32_t DDCId,double *Gain));
    G66WSB_METHOD(int,SetVolume,(G66WSB_THIS uint32_t DDCId,uint8_t Volume));
    G66WSB_METHOD(int,GetVolume,(G66WSB_THIS uint32_t DDCId,uint8_t *Volume));
    G66WSB_METHOD(int,SetMute,(G66WSB_THIS uint32_t DDCId,int Mute));
    G66WSB_METHOD(int,GetMute,(G66WSB_THIS uint32_t DDC2ChannDDCIdelId,int *Mute));

    G66WSB_METHOD(int,SetBandwidth,(G66WSB_THIS uint32_t Bandwidth));
    G66WSB_METHOD(int,GetBandwidth,(G66WSB_THIS uint32_t *Bandwidth));

    G66WSB_METHOD(int,StartChannelActivity,(G66WSB_THIS uint16_t Interval));
    G66WSB_METHOD_NO_PARAMS(int,StopChannelActivity);

    G66WSB_METHOD(int,GetSpectrumCompensation,(G66WSB_THIS uint32_t Channel,float *Value));

    G66WSB_METHOD(int,SetAudioSampleRate,(G66WSB_THIS uint32_t SampleRateIndex));
    G66WSB_METHOD(int,GetAudioSampleRate,(G66WSB_THIS uint32_t *SampleRateIndex,uint32_t *SampleRate));
};



typedef void (*G66WSB_DDC_STREAM_CALLBACK)(uint32_t DDCId,const float *Buffer,uint32_t Count,float Peak,float RMS,uintptr_t UserData);
typedef void (*G66WSB_AUDIO_STREAM_CALLBACK)(uint32_t DDCId,uint32_t Stage,const float *Buffer,uint32_t Count,uintptr_t UserData);
typedef void (*G66WSB_CHANNEL_ACTIVITY_CALLBACK)(const float *RMS,const float *dBm,uint16_t ADCLevel,uintptr_t UserData);

typedef struct
{
    G66WSB_DDC_STREAM_CALLBACK          DDCStreamCallback;
    G66WSB_AUDIO_STREAM_CALLBACK        AudioStreamCallback;
    G66WSB_CHANNEL_ACTIVITY_CALLBACK    ChannelActivityCallback;
} G66WSB_CALLBACKS;

#pragma pack(pop)

typedef int (*G66WSB_GET_DEVICE_LIST)(G66WSB_DEVICE_INFO **DeviceList,uint32_t *Count);
typedef void (*G66WSB_FREE_DEVICE_LIST)(G66WSB_DEVICE_INFO *DeviceList);
typedef int32_t (*G66WSB_OPEN_DEVICE)(const char *SerialNumber);
typedef int (*G66WSB_CLOSE_DEVICE)(int32_t hDevice);
typedef int (*G66WSB_IS_DEVICE_CONNECTED)(int32_t hDevice,int *Connected);
typedef int (*G66WSB_SET_CALLBACKS)(int32_t hDevice,const G66WSB_CALLBACKS *Callbacks,uintptr_t UserData);
typedef int (*G66WSB_GET_DEVICE_INFO)(int32_t hDevice,G66WSB_DEVICE_INFO *DeviceInfo);
typedef int (*G66WSB_SET_LED)(int32_t hDevice,uint32_t LEDMode);
typedef int (*G66WSB_GET_LED)(int32_t hDevice,uint32_t *LEDMode);
typedef int (*G66WSB_SET_POWER)(int32_t hDevice,int Power);
typedef int (*G66WSB_GET_POWER)(int32_t hDevice,int *Power);
typedef int (*G66WSB_GET_DEVICE_STATE)(int32_t hDevice,G66WSB_DEVICE_STATE *State);
typedef int (*G66WSB_SET_BUILT_IN_TEST)(int32_t hDevice,int Enabled);
typedef int (*G66WSB_GET_BUILT_IN_TEST)(int32_t hDevice,int *Enabled);
typedef int (*G66WSB_SET_ATTENUATOR)(int32_t hDevice,uint32_t Attenuator);
typedef int (*G66WSB_GET_ATTENUATOR)(int32_t hDevice,uint32_t *Attenuator);
typedef int (*G66WSB_SET_PREAMPLIFIER)(int32_t hDevice,int Preamp);
typedef int (*G66WSB_GET_PREAMPLIFIER)(int32_t hDevice,int *Preamp);
typedef int (*G66WSB_SET_ADC_NOISE_BLANKER)(int32_t hDevice,int Enabled);
typedef int (*G66WSB_GET_ADC_NOISE_BLANKER)(int32_t hDevice,int *Enabled);
typedef int (*G66WSB_SET_ADC_NOISE_BLANKER_THRESHOLD)(int32_t hDevice,uint16_t Threshold);
typedef int (*G66WSB_GET_ADC_NOISE_BLANKER_THRESHOLD)(int32_t hDevice,uint16_t *Threshold);
typedef int (*G66WSB_CREATE_DDC)(int32_t hDevice,uint32_t *DDCId);
typedef int (*G66WSB_DELETE_DDC)(int32_t hDevice,uint32_t DDCId);
typedef int (*G66WSB_START_DDC)(int32_t hDevice,uint32_t DDCId,uint32_t SampleSetsPerBuffer);
typedef int (*G66WSB_STOP_DDC)(int32_t hDevice,uint32_t DDCId);
typedef int (*G66WSB_SET_CHANNEL)(int32_t hDevice,uint32_t DDCId,uint32_t Channel);
typedef int (*G66WSB_GET_CHANNEL)(int32_t hDevice,uint32_t DDCId,uint32_t *Channel,uint32_t *Frequency);
typedef int (*G66WSB_GET_SIGNAL_LEVEL)(int32_t hDevice,uint32_t DDCId,float *Peak,float *RMS);
typedef int (*G66WSB_START_AUDIO)(int32_t hDevice,uint32_t DDCId,uint32_t SamplesPerBuffer);
typedef int (*G66WSB_STOP_AUDIO)(int32_t hDevice,uint32_t DDC2ChannelId);
typedef int (*G66WSB_SET_AUDIO_FILTER)(int32_t hDevice,uint32_t DDCId,int Enabled);
typedef int (*G66WSB_GET_AUDIO_FILTER)(int32_t hDevice,uint32_t DDCId,int *Enabled);
typedef int (*G66WSB_SET_AUDIO_FILTER_PARAMS)(int32_t hDevice,uint32_t DDCId,uint32_t CutOffLow,uint32_t CutOffHigh,double Deemphasis);
typedef int (*G66WSB_GET_AUDIO_FILTER_PARAMS)(int32_t hDevice,uint32_t DDCId,uint32_t *CutOffLow,uint32_t *CutOffHigh,double *Deemphasis);
typedef int (*G66WSB_SET_AUDIO_FILTER_LENGTH)(int32_t hDevice,uint32_t DDCId,uint32_t Length);
typedef int (*G66WSB_GET_AUDIO_FILTER_LENGTH)(int32_t hDevice,uint32_t DDCId,uint32_t *Length);
typedef int (*G66WSB_SET_AUDIO_GAIN)(int32_t hDevice,uint32_t DDCId,double Gain);
typedef int (*G66WSB_GET_AUDIO_GAIN)(int32_t hDevice,uint32_t DDCId,double *Gain);
typedef int (*G66WSB_SET_VOLUME)(int32_t hDevice,uint32_t DDCId,uint8_t Volume);
typedef int (*G66WSB_GET_VOLUME)(int32_t hDevice,uint32_t DDCId,uint8_t *Volume);
typedef int (*G66WSB_SET_MUTE)(int32_t hDevice,uint32_t DDCId,int Mute);
typedef int (*G66WSB_GET_MUTE)(int32_t hDevice,uint32_t DDCId,int *Mute);
typedef int (*G66WSB_GET_SPECTRUM_COMPENSATION)(int32_t hDevice,uint32_t Channel,float *Value);
typedef int (*G66WSB_SET_BANDWIDTH)(int32_t hDevice,uint32_t Bandwidth);
typedef int (*G66WSB_GET_BANDWIDTH)(int32_t hDevice,uint32_t *Bandwidth);
typedef int (*G66WSB_START_CHANNEL_ACTIVITY)(int32_t hDevice,uint16_t Interval);
typedef int (*G66WSB_STOP_CHANNEL_ACTIVITY)(int32_t hDevice);
typedef int (*G66WSB_GET_CHANNEL_FREQUENCY)(int32_t hDevice,uint32_t Channel,uint32_t *Frequency);
typedef int (*G66WSB_SET_AUDIO_SAMPLE_RATE)(int32_t hDevice,uint32_t SampleRateIndex);
typedef int (*G66WSB_GET_AUDIO_SAMPLE_RATE)(int32_t hDevice,uint32_t *SampleRateIndex,uint32_t *SampleRate);

#endif // __G66WSBAPI_H__

